! 29/11/2010 : New module used for computing qsat
! previously in constantes_veg
! We decide to put these routines in sechiba

MODULE qsat_moisture

  USE defprec
  USE constantes
  USE IOIPSL

!-
  IMPLICIT NONE

  !-
  LOGICAL,SAVE :: l_qsat_first=.TRUE.

  ! Size of local array to keep saturated humidity
  ! at each temperature level
  INTEGER(i_std),PARAMETER :: max_temp=370

  ! Minimum temperature for saturated humidity
  INTEGER(i_std),PARAMETER :: min_temp=100

  ! Local array to keep saturated humidity at each temperature level
  REAL(r_std),DIMENSION(max_temp),SAVE :: qsfrict

  CONTAINS
!===
    SUBROUTINE qsatcalc (kjpindex,temp_in,pres_in,qsat_out)
!---------------------------------------------------------------------
      ! input value
      ! Domain size
      INTEGER(i_std),INTENT(in) :: kjpindex
      ! Temperature in degre Kelvin
      REAL(r_std),DIMENSION(kjpindex),INTENT(in)  :: temp_in
      ! Pressure
      REAL(r_std),DIMENSION(kjpindex),INTENT(in)  :: pres_in
    ! output value
      ! Result
      REAL(r_std),DIMENSION(kjpindex),INTENT(out) :: qsat_out
      !-
    ! local variables
      INTEGER(i_std), DIMENSION(kjpindex)       :: jt
      INTEGER(i_std)                            :: ji
      REAL(r_std),DIMENSION(kjpindex)     :: zz_a, zz_b, zz_f
      INTEGER(i_std)                     :: nbad
      INTEGER(i_std),DIMENSION(1)        :: lo
!---------------------------------------------------------------------
      IF (l_qsat_first) THEN
         CALL qsfrict_init
         l_qsat_first = .FALSE.
      ENDIF
      !-
      ! 1. computes qsat interpolation into two successive temperature
      !-
      jt = INT(temp_in(:))
      !-
      nbad = COUNT(jt(:) >= max_temp-1)
      IF (nbad > 0) THEN
         WRITE(numout,*) ' qsatcalc: temperature too high at ', &
              &    nbad, ' points.'
         IF (.NOT.diag_qsat) THEN
            CALL ipslerr(2,'qsatcalc','diffuco', '', &
 &                     'temperature incorect.')
         ELSE
            lo = MAXLOC(temp_in(:))
            WRITE(numout,*) &
                 &     'Maximum temperature ( ',MAXVAL(temp_in),') found at ',lo(1)
            WHERE (jt(:) >= max_temp-1)   jt(:) = max_temp-1
         ENDIF
      ENDIF
      !-
      nbad = COUNT(jt(:) <= min_temp)
      IF (nbad > 0) THEN
         WRITE(numout,*) ' qsatcalc: temperature too low at ', &
              &    nbad, ' points.'
         IF (.NOT.diag_qsat) THEN
            CALL ipslerr(2,'qsatcalc','diffuco', '', &
                 &                   'temperature incorect.')
         ELSE
            lo = MINLOC(temp_in(:))
            WRITE(numout,*) &
                 &     'Minimum temperature ( ',MINVAL(temp_in),') found at ',lo(1)
            WHERE (jt(:) <= min_temp)   jt(:) = min_temp
         ENDIF
      ENDIF
      !-
      DO ji = 1, kjpindex
         zz_f(ji) = temp_in(ji)-FLOAT(jt(ji))
         zz_a(ji) = qsfrict(jt(ji))
         zz_b(ji) = qsfrict(jt(ji)+1)
      ENDDO
      !-
      ! 2. interpolates between this two values
      !-
      DO ji = 1, kjpindex
         qsat_out(ji) = ((zz_b(ji)-zz_a(ji))*zz_f(ji)+zz_a(ji))/pres_in(ji)
      ENDDO
      !----------------------
    END SUBROUTINE qsatcalc
    !===
    FUNCTION qsat (temp_in,pres_in) RESULT (qsat_result)
      !!--------------------------------------------------------------------
      !! FUNCTION qsat (temp_in, pres_in) RESULT (qsat_result)
      !!--------------------------------------------------------------------
      REAL(r_std),INTENT(in) :: temp_in  ! Temperature in degre Kelvin
      REAL(r_std),INTENT(in) :: pres_in  ! Pressure
      REAL(r_std) :: qsat_result
      !-
      INTEGER(i_std)        :: jt
      REAL(r_std)     :: zz_a,zz_b,zz_f
      !---------------------------------------------------------------------
      IF (l_qsat_first) THEN
         CALL qsfrict_init
         l_qsat_first = .FALSE.
      ENDIF
      !-
      ! 1. computes qsat interpolation into two successive temperature
      !-
      jt = INT(temp_in)
      !-
      IF (jt >= max_temp-1) THEN
         WRITE(numout,*) &
              &   ' We stop. temperature too BIG : ',temp_in, &
              &   ' approximation for : ',jt
         IF (.NOT.diag_qsat) THEN
            CALL ipslerr(2,'qsat','', '',&
                 &                   'temperature incorect.')
         ELSE
            qsat_result = 999999.
            RETURN
         ENDIF
      ENDIF
      !-
      IF (jt <= min_temp ) THEN
         WRITE(numout,*) &
              &   ' We stop. temperature too SMALL : ',temp_in, &
              &   ' approximation for : ',jt
         IF (.NOT.diag_qsat) THEN
            CALL ipslerr(2,'qsat','', '',&
                 &                   'temperature incorect.')
         ELSE
            qsat_result = -999999.
            RETURN
         ENDIF
      ENDIF
      !-
      zz_f = temp_in-FLOAT(jt)
      zz_a = qsfrict(jt)
      zz_b = qsfrict(jt+1)
      !-
      ! 2. interpolates between this two values
      !-
      qsat_result = ((zz_b-zz_a)*zz_f+zz_a)/pres_in
      !----------------
    END FUNCTION qsat
    !===
    SUBROUTINE dev_qsatcalc (kjpindex,temp_in,pres_in,dev_qsat_out)
      !---------------------------------------------------------------------
      ! Domain size
      INTEGER(i_std),INTENT(in)                  :: kjpindex
      ! Temperature in degre Kelvin
      REAL(r_std),DIMENSION(kjpindex),INTENT(in)  :: temp_in
      ! Pressure
      REAL(r_std),DIMENSION(kjpindex),INTENT(in)  :: pres_in
      ! Result
      REAL(r_std),DIMENSION(kjpindex),INTENT(out) :: dev_qsat_out
      !-
      INTEGER(i_std),DIMENSION(kjpindex) :: jt
      INTEGER(i_std)                     :: ji
      REAL(r_std),DIMENSION(kjpindex)     :: zz_a, zz_b, zz_c, zz_f
      INTEGER(i_std)                     :: nbad
      !---------------------------------------------------------------------
      IF (l_qsat_first) THEN
         CALL qsfrict_init
         l_qsat_first = .FALSE.
      ENDIF
      !-
      ! 1. computes qsat interpolation into two successive temperature
      !-
      jt = INT(temp_in(:)+undemi)
      !-
      nbad = COUNT( jt(:) >= max_temp-1 )
      IF (nbad > 0) THEN
         WRITE(numout,*) &
              &   ' dev_qsatcalc: temperature too high at ',nbad,' points.'
         IF (.NOT.diag_qsat) THEN
            CALL ipslerr(3,'dev_qsatcalc','', '', &
                 &                   'temperature incorect.')
         ELSE
            WHERE (jt(:) >= max_temp-1)   jt(:) = max_temp-1
         ENDIF
      ENDIF
      !-
      nbad = COUNT( jt(:) <= min_temp )
      IF (nbad > 0) THEN
         WRITE(numout,*) &
              &   ' dev_qsatcalc: temperature too low at ',nbad,' points.'
         IF (.NOT.diag_qsat) THEN
            CALL ipslerr(3,'dev_qsatcalc', '', '',&
                 &                   'temperature incorect.')
         ELSE
            WHERE (jt(:) <= min_temp)   jt(:) = min_temp
         ENDIF
      ENDIF
      !-
      DO ji=1,kjpindex
         zz_f(ji) = temp_in(ji)+undemi-FLOAT(jt(ji))
         zz_a(ji) = qsfrict(jt(ji)-1)
         zz_b(ji) = qsfrict(jt(ji))
         zz_c(ji) = qsfrict(jt(ji)+1)
      ENDDO
      !-
      ! 2. interpolates between this two values
      !-
      DO ji = 1, kjpindex
         dev_qsat_out(ji) = &
              &   ((zz_c(ji)-deux*zz_b(ji)+zz_a(ji))*(zz_f(ji)-un) + &
              &                         zz_c(ji)-zz_b(ji))/pres_in(ji)
      ENDDO
      !--------------------------
    END SUBROUTINE dev_qsatcalc
    !===
    FUNCTION dev_qsat (temp_in,pres_in) RESULT (dev_qsat_result)
      !!--------------------------------------------------------------------
      !! FUNCTION dev_qsat (temp_in, pres_in) RESULT (dev_qsat_result)
      !! computes deviation of qsat
      !!--------------------------------------------------------------------
      REAL(r_std),INTENT(in)  :: pres_in    ! Pressure
      REAL(r_std),INTENT(in)  :: temp_in    ! Temperture in degre Kelvin
      REAL(r_std) :: dev_qsat_result
      !-
      INTEGER(i_std)        :: jt
      REAL(r_std)     :: zz_a, zz_b, zz_c, zz_f
      !---------------------------------------------------------------------
      IF (l_qsat_first) THEN
         CALL qsfrict_init
         l_qsat_first = .FALSE.
      ENDIF
      !-
      ! 1. computes qsat deviation interpolation
      !    into two successive temperature
      !-
      jt = INT(temp_in+undemi)
      !-
      IF (jt >= max_temp-1) THEN
         WRITE(numout,*) &
              &   ' We stop. temperature too HIGH : ',temp_in, &
              &   ' approximation for : ',jt
         IF (.NOT.diag_qsat) THEN
            CALL ipslerr(3,'dev_qsat','', '',&
                 &                   'temperature incorect.')
         ELSE
            dev_qsat_result = 999999.
            RETURN
         ENDIF
      ENDIF
      !-
      IF (jt <= min_temp ) THEN
         WRITE(numout,*) &
              &   ' We stop. temperature too LOW : ',temp_in, &
              &   ' approximation for : ',jt
         IF (.NOT.diag_qsat) THEN
            CALL ipslerr(3,'dev_qsat','', '',&
                 &                    'temperature incorect.')
         ELSE
            dev_qsat_result = -999999.
            RETURN
         ENDIF
      ENDIF
      !-
      zz_f = temp_in+undemi-FLOAT(jt)
      zz_a = qsfrict(jt-1)
      zz_b = qsfrict(jt)
      zz_c = qsfrict(jt+1)
      !-
      ! 2. interpolates
      !-
      dev_qsat_result=((zz_c-deux*zz_b+zz_a)*(zz_f-un)+zz_c-zz_b)/pres_in
      !--------------------
    END FUNCTION dev_qsat
    !===
    SUBROUTINE qsfrict_init
      !!--------------------------------------------------------------------
      !! The qsfrict_init routine initialises qsfrict array
      !! to store precalculated value for qsat
      !!--------------------------------------------------------------------
      INTEGER(i_std)        :: ji
      REAL(r_std)     :: zrapp,zcorr,ztemperature,zqsat
      !---------------------------------------------------------------------
      ! initialisation
      zrapp = msmlr_h2o/msmlr_air
      zcorr = 0.00320991_r_std
      ! computes saturated humidity one time and store in qsfrict local array
      DO ji=100,max_temp
         ztemperature = FLOAT(ji)
         IF (ztemperature < 273._r_std) THEN
            zqsat = zrapp*10.0_r_std**(2.07023_r_std-zcorr*ztemperature &
                 &             -2484.896/ztemperature+3.56654*LOG10(ztemperature))
         ELSE
            zqsat = zrapp*10.0**(23.8319-2948.964/ztemperature &
                 &              -5.028*LOG10(ztemperature) &
                 &              -29810.16*EXP(-0.0699382*ztemperature) &
                 &              +25.21935*EXP(-2999.924/ztemperature))
         ENDIF
         qsfrict (ji) = zqsat
      ENDDO
      !-
      qsfrict(1:100) = zero
      !-
      IF (long_print) WRITE (numout,*) ' qsfrict_init done'
      !--------------------------
    END SUBROUTINE qsfrict_init



END MODULE qsat_moisture
